% For joint RP-ABS and CSB optimization (FeICIC)
% Only pico BSs perform CSB optimization, therefore:
% 1. If a pico BS is chosen to do the joint optimization, the CSB value of
%    the pico BS and the RP-ABS pattern of the macro BS in the same hexagon 
%    will be optimized.
% 2. If a macro BS is chosen to do the joint optimization, only the RP-ABS
%    pattern of the macro BS will be optimized

% The value of Scheduler must be one of the followings:
% 'RR': Means round-robin
% 'PF_LTE': Means proportional fair using the one proposed in LTE reference
%           book "LTE - The UMTS Long Term Evolution: From Theory to
%           Practice", 2009, pp. 291
% 'CC': Means cake-cutting

% Ix: Index of the station who is finding the best stragegy
% pf_window: Window size of the LTE PF scheduler

% Reduced-power ABS is utilized for each PRB that belongs to an ABS.

function [Station,UE] = BetterResp_FeICIC(Station,Ix,UE,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,Scheduler,pf_window,power_Opt_method,ideal)

% Find the macro BS in the same hexagon, if Station(Ix) is a pico BS
if Station(Ix).Type == 1
    IxMacro = Ix;
    CSB_value = 0; % Do not perform CSB optimization if the BS is a macro BS
else
    profile = [[Station.Type]' [Station.ID]' [Station.Cluster]'];
    [nouse,IxMacro] = ismember([1 Station(Ix).MacroID 1],profile,'rows');
end

BestCSB = Station(Ix).CSB;
BestABS = Station(IxMacro).ABS; % It represents RP-ABS, 1 means transmit at full power
OriginalABS = Station(IxMacro).ABS;
AffectedUEix_best = cell(length(Station(Ix).AllNeighbor)+1,1);
AffectedUEix_best{1} = find([UE.attach]==Station(Ix).NameTag);
for k = 1:length(Station(Ix).AllNeighbor)
    ix = find([Station.NameTag]==Station(Ix).AllNeighbor(k));
    AffectedUEix_best{k+1} = find([UE.attach]==Station(ix).NameTag);
end
BestUtility = sum(log([UE([AffectedUEix_best{:}]).throughput]));
NumPRB = NumTimeSlots*NumFreqSlots;

% Check how many different user attachment patterns (w.r.t. CSB) are there
Station(Ix).CSB = 0;
[Station,UE] = update_status(Station,Ix,UE);
UE_last_CSB = Pilot_CSB_Asso(UE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
CSB_value_meaningful = zeros(1,length(CSB_value)); % Record the CSB values that lead to changes in user attachment
count = 1;
for i = 2:length(CSB_value) % Skip the first CSB value because it is zero and it has been considered
    Station(Ix).CSB = CSB_value(i);
    [Station,UE] = update_status(Station,Ix,UE);
    UE = Pilot_CSB_Asso(UE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    if ~isequal([UE_last_CSB.attach],[UE.attach])
        count = count + 1;
        CSB_value_meaningful(count) = CSB_value(i);
        UE_last_CSB = UE;
    end  
end
if count < length(CSB_value)
    CSB_value_meaningful(count+1:end) = [];
end

% Do optimization for each of the CSB value that changes user attachment
for i = 1:length(CSB_value_meaningful)
    Station(Ix).CSB = CSB_value_meaningful(i);
    [Station,UE] = update_status(Station,Ix,UE);
    % Update UE attachment
    UE = Pilot_CSB_Asso(UE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    % Reset the list of UEs that are affected by the player
    AffectedUEix = cell(length(Station(Ix).AllNeighbor)+1,1);
    AffectedUEix{1} = find([UE.attach]==Station(Ix).NameTag);
    for k = 1:length(Station(Ix).AllNeighbor)
        ix = find([Station.NameTag]==Station(Ix).AllNeighbor(k));
        AffectedUEix{k+1} = find([UE.attach]==Station(ix).NameTag);
    end
    
    % Reset the transmission power of the macro BS in the same hexagon as
    % Station(Ix) to the original RP-ABS pattern
    Station(IxMacro).ABS = OriginalABS;
    [Station,UE] = update_status(Station,IxMacro,UE);
    % Perform scheduling
    [Station,UE] = Schedule_all(Station,Ix,UE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,Scheduler,pf_window);
    % Calculate throughput of the involved users
    UE(AffectedUEix{1}) = Throughput(Station(Ix).NameTag,UE(AffectedUEix{1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    for k = 1:length(AffectedUEix)-1
        UE(AffectedUEix{k+1}) = Throughput(Station(Ix).AllNeighbor(k),UE(AffectedUEix{k+1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    end
    % We adjust power on the PRBs that belong to ABSs by referring to the
    % ABS pattern that provides the largest number of ABSs.
    % Find the PRBs that belong to ABSs in the ABS pattern that has the
    % largest number of ABSs
    RP_PRB = abs(repmat(ABSset(NumABS,:),[1 NumFreqSlots])-1);
    RP_PRB_ix = find(RP_PRB);
    % For each PRB that is subject to power optimization
    for p = 1:length(RP_PRB_ix)
        ThisPRB_ix = RP_PRB_ix(p);
        % Find the affected users on this PRB
        UEix_ThisPRB_ThisMacro = Station(IxMacro).UEinPRB(ThisPRB_ix);
        
        if ideal == 1
            OtherUE_list = Station(IxMacro).Interfering;
        elseif ideal == 0
            OtherUE_list = Station(IxMacro).InterferingIdeal;
        end
        OtherUE_list(find(OtherUE_list==UEix_ThisPRB_ThisMacro)) = [];
        OtherUE_Interfered_Allocation = reshape([UE(OtherUE_list).Allocation],NumPRB,[])';
        OtherUE_Interfered_Allocation(find(OtherUE_list==UEix_ThisPRB_ThisMacro),:) = [];
        UEinPRB_OtherBS = OtherUE_list(find(OtherUE_Interfered_Allocation(:,ThisPRB_ix)));

        % Find out the variables for the power optimization problem
        SINR_ThisPRB_ThisMacro = UE(UEix_ThisPRB_ThisMacro).SINR_PerPRB_FullPower(ThisPRB_ix);
        Rate_ThisUE_OtherPRB = sum(UE(UEix_ThisPRB_ThisMacro).ThroughputPerPRB.*[Station(IxMacro).UEinPRB==UEix_ThisPRB_ThisMacro]) ...
            -UE(UEix_ThisPRB_ThisMacro).ThroughputPerPRB(ThisPRB_ix); % The rate of the UE in the current PRB of Station(IxMacro) that is achieved on other PRBs
        
        RatePerPRB_OtherUE = reshape([UE(UEinPRB_OtherBS).Allocation].*[UE(UEinPRB_OtherBS).ThroughputPerPRB],[NumPRB,length(UEinPRB_OtherBS)])';
        Rate_OtherUE_OtherPRB = sum(RatePerPRB_OtherUE,2)-RatePerPRB_OtherUE(:,ThisPRB_ix);
        InfPower_OtherUE_Matrix = [UE(UEinPRB_OtherBS).RSS];
        InfPower_OtherUE_ThisPRB_Matrix = InfPower_OtherUE_Matrix(:,ThisPRB_ix+[0:NumPRB:NumPRB*(length(UEinPRB_OtherBS)-1)]);
        TotalInfPower_OtherUE_ThisPRB = sum(InfPower_OtherUE_ThisPRB_Matrix,1);
        IntPower_OtherUE_ThisMacro = InfPower_OtherUE_ThisPRB_Matrix([UE(UEinPRB_OtherBS).InterfereIx]==IxMacro);
        [nouse,OtherUE_ServingBS_Ix] = ismember([UE(UEinPRB_OtherBS).attach],[Station.NameTag]);
        ServePower_OtherUE = InfPower_OtherUE_ThisPRB_Matrix( repmat(OtherUE_ServingBS_Ix,length(UE(1).InterfereIx),1)==[UE(UEinPRB_OtherBS).InterfereIx] );
        IntPower_OtherUE_OtherBS = TotalInfPower_OtherUE_ThisPRB' - IntPower_OtherUE_ThisMacro ...
            - ServePower_OtherUE + 10^(NoiseDen/10)/1000*BWperRB * 10^(NoiseFigure/10);
        % Perform power optimization for that PRB
        [Station(IxMacro).ABS(RP_PRB_ix(p))] = PerPRB_Power_Optimization...
            (SINR_ThisPRB_ThisMacro,Rate_ThisUE_OtherPRB,ServePower_OtherUE,IntPower_OtherUE_ThisMacro,IntPower_OtherUE_OtherBS,Rate_OtherUE_OtherPRB,power_Opt_method);
        % Avoid the case of transmitting zero power in order to avoid
        % the situation where no user is scheduled in a PRB which then
        % makes UEix_ThisPRB_ThisMacro=0 and create bug
        if Station(IxMacro).ABS(RP_PRB_ix(p)) < 10^-10
            Station(IxMacro).ABS(RP_PRB_ix(p)) = 10^-10;
        end
        % Update the users' throughput on that PRB
        [Station,UE] = update_status(Station,IxMacro,UE);
        UE(UEix_ThisPRB_ThisMacro) = Throughput(Station(IxMacro).NameTag,UE(UEix_ThisPRB_ThisMacro),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        for k = 1:length(UEinPRB_OtherBS)
            UE(UEinPRB_OtherBS(k)) = Throughput(UE(UEinPRB_OtherBS(k)).attach,UE(UEinPRB_OtherBS(k)),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        end
    end
    % Perform scheduling for all affected stations
    [Station,UE] = Schedule_all(Station,IxMacro,UE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,Scheduler,pf_window);
    % Update UE throughput
    UE(AffectedUEix{1}) = Throughput(Station(Ix).NameTag,UE(AffectedUEix{1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    for k = 1:length(AffectedUEix)-1
        UE(AffectedUEix{k+1}) = Throughput(Station(Ix).AllNeighbor(k),UE(AffectedUEix{k+1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    end
    % Find out the utility of the player, i.e., sum of log rates of users
    % who are attached to the player and the neighbours of the player
    Utility = sum(log([UE([AffectedUEix{:}]).throughput]));
    % Update the memory on setting if the current setting is better
    if Utility > BestUtility
        BestUtility = Utility;
        BestCSB = CSB_value_meaningful(i);
        BestABS = Station(IxMacro).ABS;
        AffectedUEix_best = AffectedUEix;
    end
end

% Adopt the better response
Station(Ix).CSB = BestCSB;
[Station,UE] = update_status(Station,Ix,UE);
UE = Pilot_CSB_Asso(UE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
Station(IxMacro).ABS = BestABS;
[Station,UE] = update_status(Station,IxMacro,UE);
[Station,UE] = Schedule_all(Station,IxMacro,UE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,Scheduler,pf_window);
UE(AffectedUEix_best{1}) = Throughput(Station(Ix).NameTag,UE(AffectedUEix_best{1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
for k = 1:length(AffectedUEix_best)-1
    UE(AffectedUEix_best{k+1}) = Throughput(Station(Ix).AllNeighbor(k),UE(AffectedUEix_best{k+1}),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
end

% to_remove = who;
% to_remove = strrep(to_remove,'Station','');
% to_remove = strrep(to_remove,'UE','');
% clear(to_remove{:})
% 
% pause(5)