% find an augmentation path in a trellis graph
% the trellis graph is partitioned into 4 levels, there are only links between adjacent level
% the first level consists of a single source
% the fourth level consists of a single sink
% the second level has N vertices
% the third level has Q vertices
%
% the vertices in the second level connected to the source is stored in vector a
% the vertices in the third level connected to the sink is stored in vector b
% forward is an NxQ matrix representing forward links from level 2 to 3
% backward is an NxQ matrix representing backward links from level 3 to 2

function [aug_path, A, B ]= augmentation_path(a,forward,backward,b)
%  aug_path is an augmentation_path if it exists
%
%  if the source cannot be reached from the source, then aug_path = []
%  in this case, A is the reachable vertices in level 2
%  and B is the reachable vertices in level 3
%  when aug_path is not [], A and B are incomplete list of reachable vertices



[N Q] = size(forward);
aug_path = [];              % initialized to an empty vector
alpha = zeros(1,N);
alpha(a) = ones(size(a));   % indicator vector of a  ( row vector)
beta = zeros(1,Q);
beta(b) = ones(size(b));    % indicator vector of b  ( col vector)

label2 = alpha-1;  % label2(i) = 0 if vertex i can be reached from the source, -1 otherwise
label3 = repmat([-1],1,Q);    % initialized the label in level 3 to -1

list = a;     % the list of vectices reachable from the source
pointer = 0;

while (pointer < length(list))
	pointer = pointer+1;
	n = list(pointer);   % a node in the list
	adj = forward(n,:);  % adj(i) is 1 iff i is adjacent to n
	x = find( ( max(-label3,0) & adj ) > 0); % find the not yet labeled vertices in adj
	if (~isempty(x))
      label3(x) = n*ones(size(x));   % recorded that the vertices in adj can be reached from n
	end
	
	if (beta*adj' > 0)  % if the sink vertex is reachable
      f = find((beta & adj)==1); 
      if (isempty(f))   % error check
          error('error in finding augmentation path')
      end
      
      %aug_path = [n f(randint(1,1,length(f))+1)];   % pick a random path from f
      aug_path = [n f(1)];   % pick a path with smaller index
      while( label2(n) ~= 0)    % backtrack and construct an augmentation path
         g = label2(n);
         n = label3(g);
         aug_path = [[n g] aug_path];
      end
      break;
	else
       index = find(adj);
       for j = index
         t= backward(:,j)';
         if (max(t & max(-label2,0)) > 0); % add some reachable vertices in level 2 to the list
            x = find (t & max(-label2,0));
            if (isempty(x))  % error check
                error('error in augmentation_path: empty x')
            end
            label2(x) = j*ones(size(x));  % update label2
            list = [list x];  % add x to the end of list
         end
       end
	end
end

A = find(label2>=0);   % vertices that are reachable from the sources
B = find(label3>=0);
