%% This is part of the simulation program for the following paper
%% Y. Liu, C. S. Chen, C. W. Sung and C. Singh, "A Game Theoretic Distributed Algorithm for FeICIC Optimization in LTE-A HetNets," 
%% in IEEE/ACM Transactions on Networking, vol. 25, no. 6, pp. 3500-3513, Dec. 2017. doi: 10.1109/TNET.2017.2748567

%% Please site the above paper if you use any part of this program

%% The author of the program is Ye Liu (yeliu27-c@my.cityu.edu.hk)

%% Special thanks to Kenneth Shum for providing the codes for the cake-cutting algorithm

%=========================================================================%

%% The main program of the distributed eICIC/FeICIC optimization using potential game

%% This program considers the non-exact potential game formulation, see Table III of the above paper:

%% Some tip for how to run the codes:
%% To avoid errors like "out of memory", it is recommended to run the codes 
%% in many MATLAB sessions, where only a few random topologies are simulated 
%% in each MATLAB session

% Abbreviations:
% GU: global utility
% CCP: convex concave procedure
% RR: round-robin
% PF: proportional fair

clc
clear

ideal = 0; % Whether the game is exact potential or not (by changing the interference range of a station, see Theorem 2)
if ideal ~= 1 && ideal ~= 0
    error('The value of the variable ideal is not correct!')
end

NumeNBPerCluster = 19; % Please keep this fixed at 19, other values are not tested and may lead to error
NumPicoPerCell = 2;
NumUEPerCell = 8;
NumTimeSlots = 10;
NumFreqSlots = 2;
NumPRB = NumTimeSlots*NumFreqSlots;
UE_near_Pico_Fraction = 0.5; % Define the fraction of all users in a cell who are located near the pico BSs in the same cell
BWperRB = 180 * 1000; % Bandwidth per resource block

% Define the set of ABS patterns, 0 means ABS
ABSset = [1 1 1 1 1 1 1 1 1 1; % Number of columns must be equal to NumTimeSlots
    0 1 1 1 1 1 1 1 1 1;
    0 1 1 1 1 0 1 1 1 1;
    0 1 0 1 1 0 1 1 1 1;
    0 1 0 1 1 0 1 0 1 1;
    0 1 0 1 0 0 1 0 1 1;
    0 1 0 1 0 0 1 0 1 0;];
NumABS = size(ABSset,1);
CSB_value = [0 3 6 9 12 15]; % All possible CSB values in dB

% The base stations in the center cluster
NumStationIcare = NumeNBPerCluster + NumeNBPerCluster * NumPicoPerCell;

% This is for saving data
% To speed up simulation, one may consider to use multiple MATLABs to run
% this code, where each MATLAB should use a different ID
[~,ComputerName] = system('hostname');
ID = 3;
if ideal == 1
    FileName = strcat('FeICIC_ideal_',int2str(NumPicoPerCell),'pico_',ComputerName,'_',int2str(ID),'.mat');
elseif ideal == 0
    FileName = strcat('FeICIC_nonideal_',int2str(NumPicoPerCell),'pico_',ComputerName,'_',int2str(ID),'.mat');
end

Topology = 5; % Number of topologies
MaxIteration = 300; % Maximum number of iterations allowed for best/better response dynamics
Performed_iteration = zeros(Topology,1);

% To record the cell-edge throughput
% RR: The round-robin scheduling
% PF_LTE: The proportional-fair scheduling as defined in Section VI-B
%         of the reference paper
% CC: The cake-cutting scheduler
cell_edge_rate_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);            % No optimization 
cell_edge_rate_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);        % No optimization
cell_edge_rate_CC = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);            % No optimization
cell_edge_rate_eICIC_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);      % eICIC
cell_edge_rate_eICIC_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);
cell_edge_rate_FeICIC_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);     % FeICIC
cell_edge_rate_FeICIC_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);
cell_edge_rate_FeICIC_CC = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);

% To record throughput of all UEs in the center cluster
rate_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);            % No optimization 
rate_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);        % No optimization
rate_CC = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);            % No optimization
rate_eICIC_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);      % eICIC
rate_eICIC_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);
rate_FeICIC_RR = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);     % FeICIC
rate_FeICIC_PF_LTE = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);
rate_FeICIC_CC = zeros(Topology*NumUEPerCell*NumeNBPerCluster,1);

% To record the total consumed energy for serving the UEs in the center
% cluster
energy_RR = zeros(Topology,1);
energy_PF_LTE = zeros(Topology,1);
energy_CC = zeros(Topology,1);
energy_eICIC_RR = zeros(Topology,1);
energy_eICIC_PF_LTE = zeros(Topology,1);
energy_FeICIC_RR = zeros(Topology,1);
energy_FeICIC_PF_LTE = zeros(Topology,1);
energy_FeICIC_CC = zeros(Topology,1);

GU_FeICIC_RR = zeros(MaxIteration,Topology); % Global utility memory for joint optimizing RP-ABS and CSB using round-robin scheduler
GU_FeICIC_PF_LTE = zeros(MaxIteration,Topology); % Global utility memory for joint optimizing RP-ABS and CSB using LTE proportional fair scheduler
GU_FeICIC_CC = zeros(MaxIteration,Topology); % Global utility memory for joint optimizing RP-ABS and CSB using cake-cutting scheduler
GU_eICIC_RR = zeros(MaxIteration,Topology); % Global utility memory for joint optimizing ABS and CSB using round-robin scheduler
GU_eICIC_PF_LTE = zeros(MaxIteration,Topology); % Global utility memory for joint optimizing ABS and CSB using LTE proportional fair scheduler

topology = 0;

start_time = datestr(clock);
timer = tic;
cell_edge_count = 1;
while topology < Topology
    
    topology = topology + 1;
    
    % Generate a network topology
    if ideal == 1
        [NumUE,NoiseDen,NoiseFigure,BWperRB,NumeNB,Station,eNB,Pico,UE]...
            = wrap_around_model_random(NumeNBPerCluster,NumPicoPerCell,NumUEPerCell,NumTimeSlots,NumFreqSlots,UE_near_Pico_Fraction);
    elseif ideal == 0
        [NumUE,NoiseDen,NoiseFigure,BWperRB,NumeNB,Station,eNB,Pico,UE]...
            = wrap_around_model_random_3tier_interference(NumeNBPerCluster,NumPicoPerCell,NumUEPerCell,NumTimeSlots,NumFreqSlots,UE_near_Pico_Fraction);
    end
    
    % Index of users in the center cluster
    UE_ix_center = find([UE.Cluster]==1);
    NumCellEdgeUE = round( 0.05 * length(UE_ix_center) );
   
    % Update the ABS/CSB patterns
    for i = 1:NumeNB
        [eNB,UE] = update_status(eNB,i,UE);
    end
    
    %% Replicate the structures Station and UE for various optimization schemes 
    % For RP-ABS and CSB joint optimization
    Station_FeICIC_RR = Station;
    UE_FeICIC_RR = UE;
    Station_FeICIC_PF_LTE = Station;
    UE_FeICIC_PF_LTE = UE;
    Station_FeICIC_CC = Station;
    UE_FeICIC_CC = UE;
    
    % For ABS and CSB joint optimization
    Station_eICIC_RR = Station; % Round-robin scheduling
    UE_eICIC_RR = UE;           % Round-robin scheduling
    Station_eICIC_PF_LTE = Station; % Proportional fair scheduling using the LTE PF scheduler
    UE_eICIC_PF_LTE = UE;           % Proportional fair scheduling using the LTE PF scheduler
    
    % No optimization
    UE_RR = UE;
    Station_RR = Station;
    UE_PF_LTE = UE;
    Station_PF_LTE = Station;
    UE_CC = UE;
    Station_CC = Station;
    
    % Record the time elapsed in each round of game play
    TimePerIt_FeICIC_CC = zeros(1,MaxIteration);
    TimePerIt_FeICIC_PF_LTE = zeros(1,MaxIteration);
    TimePerIt_FeICIC_RR = zeros(1,MaxIteration);
    TimePerIt_eICIC_PF_LTE = zeros(1,MaxIteration);
    TimePerIt_eICIC_RR = zeros(1,MaxIteration);
    
    %% Find out the list of players
    PlayerList = [Pico(1:NumPicoPerCell*NumeNBPerCluster).NameTag]';
    PlayerIx = zeros(length(PlayerList),1);
    for i = 1:length(PlayerList)
        PlayerIx(i) = find([Station.NameTag]==PlayerList(i));
    end
    
    %% Perform initial association, resource allocation, and calculate UE throughput
    % Initial association
    UE_FeICIC_RR = Pilot_CSB_Asso(UE_FeICIC_RR,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_FeICIC_PF_LTE = Pilot_CSB_Asso(UE_FeICIC_PF_LTE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_FeICIC_CC = Pilot_CSB_Asso(UE_FeICIC_CC,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_eICIC_RR = Pilot_CSB_Asso(UE_eICIC_RR,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_eICIC_PF_LTE = Pilot_CSB_Asso(UE_eICIC_PF_LTE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_RR = Pilot_CSB_Asso(UE_RR,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_PF_LTE = Pilot_CSB_Asso(UE_PF_LTE,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UE_CC = Pilot_CSB_Asso(UE_CC,NumUE,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    
    % Initial resource allocation and throughput calculation
    for i = 1:length(Station)
        ix = i;
        
        [Station_RR(ix),UE_RR,attach_RR] = ...
            round_robin_scheduler(Station_RR(ix),UE_RR,NumTimeSlots,NumFreqSlots);
        [UE_RR(attach_RR)] = ...
            Throughput(Station_RR(ix).NameTag,UE_RR(attach_RR),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_PF_LTE(ix),UE_PF_LTE,attach_PF_LTE] = ...
            PF_scheduler_LTE(Station_PF_LTE(ix),UE_PF_LTE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,10);
        [UE_PF_LTE(attach_PF_LTE)] = ...
            Throughput(Station_PF_LTE(ix).NameTag,UE_PF_LTE(attach_PF_LTE),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_CC(ix),UE_CC,attach_CC] = ...
            PF_scheduler_Kenneth(Station_CC(ix),UE_CC,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        [UE_CC(attach_CC)] = ...
            Throughput(Station_CC(ix).NameTag,UE_CC(attach_CC),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_FeICIC_CC(ix),UE_FeICIC_CC,attach_FeICIC_CC] = ...
            PF_scheduler_Kenneth(Station_FeICIC_CC(ix),UE_FeICIC_CC,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        [UE_FeICIC_CC(attach_FeICIC_CC)] = ...
            Throughput(Station_FeICIC_CC(ix).NameTag,UE_FeICIC_CC(attach_FeICIC_CC),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_FeICIC_PF_LTE(ix),UE_FeICIC_PF_LTE,attach_FeICIC_PF_LTE] = ...
            PF_scheduler_LTE(Station_FeICIC_PF_LTE(ix),UE_FeICIC_PF_LTE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,10);
        [UE_FeICIC_PF_LTE(attach_FeICIC_PF_LTE)] = ...
            Throughput(Station_FeICIC_PF_LTE(ix).NameTag,UE_FeICIC_PF_LTE(attach_FeICIC_PF_LTE),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_FeICIC_RR(ix),UE_FeICIC_RR,attach_Joint_power_CCP_RR] = ...
            round_robin_scheduler(Station_FeICIC_RR(ix),UE_FeICIC_RR,NumTimeSlots,NumFreqSlots);
        [UE_FeICIC_RR(attach_Joint_power_CCP_RR)] = ...
            Throughput(Station_FeICIC_RR(ix).NameTag,UE_FeICIC_RR(attach_Joint_power_CCP_RR),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_eICIC_RR(ix),UE_eICIC_RR,attach_Joint_RR] = ...
            round_robin_scheduler(Station_eICIC_RR(ix),UE_eICIC_RR,NumTimeSlots,NumFreqSlots);
        [UE_eICIC_RR(attach_Joint_RR)] = ...
            Throughput(Station_eICIC_RR(ix).NameTag,UE_eICIC_RR(attach_Joint_RR),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
        
        [Station_eICIC_PF_LTE(ix),UE_eICIC_PF_LTE,attach_eICIC_PF_LTE] = ...
            PF_scheduler_LTE(Station_eICIC_PF_LTE(ix),UE_eICIC_PF_LTE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,10);
        [UE_eICIC_PF_LTE(attach_eICIC_PF_LTE)] = ...
            Throughput(Station_eICIC_PF_LTE(ix).NameTag,UE_eICIC_PF_LTE(attach_eICIC_PF_LTE),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    end
        
    it = 1; % Number of iterations performed
    GU_FeICIC_RR(it,topology) = sum(log([UE_FeICIC_RR.throughput]));
    GU_FeICIC_PF_LTE(it,topology) = sum(log([UE_FeICIC_PF_LTE.throughput]));
    GU_FeICIC_CC(it,topology) = sum(log([UE_FeICIC_CC.throughput]));
    
    GU_eICIC_RR(it,topology) = sum(log([UE_eICIC_RR.throughput]));
    GU_eICIC_PF_LTE(it,topology) = sum(log([UE_eICIC_PF_LTE.throughput]));
    
    ChosenPlayerIx = zeros(MaxIteration,1); % Record the sequence of chosen players' indices
    timer_start = tic;
    timer_per_round = timer_start;
    last_round_station_type = [];
    choice_list = [0; ceil(rand(MaxIteration,1) * length(PlayerList))];
    % To avoid performing optimization for the same station consecutively
    choice_list_temp = [choice_list(2:end); 0];
    while sum(choice_list_temp==choice_list)
        choice_list(choice_list_temp==choice_list) = ceil(rand(sum(choice_list_temp==choice_list),1) * length(PlayerList));
        choice_list_temp = [choice_list(2:end); 0];
    end
    
    %% Perform the eICIC/FeICIC optimization
    while it < MaxIteration
        clc
        it = it + 1;
        Performed_iteration(topology) = it;
        start_time
        if ideal
            disp('Ideal interference range.')
        elseif ~ideal
            disp('Non-ideal interference range.')
        end
        disp(['Will simulate ',num2str(Topology),' random topologies.'])
        NumPicoPerCell
        UE_near_Pico_Fraction
        ID
        topology
        it
        last_round_station_type
        time_last_round_of_optimization = toc(timer_per_round);
        time_last_round_of_optimization
        time_from_beginning_of_optimization_of_this_topology = toc(timer_start);
        time_from_beginning_of_optimization_of_this_topology
        time_average_per_round_of_this_topology = time_from_beginning_of_optimization_of_this_topology / (it-2);
        time_average_per_round_of_this_topology
        time_from_beginning = toc(timer);
        time_from_beginning
        
        
        timer_per_round = tic;
        %% Choose a player randomly, and update the player's stragegy by best/better response
        choice = choice_list(it)
        ix = find([Station.NameTag]==PlayerList(choice));
        ChosenPlayerIx(it) = ix;
        
        timer_FeICIC_RR = tic;
        % CCP stands for concave-convex procedure
        [Station_FeICIC_RR,UE_FeICIC_RR] = BetterResp_FeICIC...
            (Station_FeICIC_RR,ix,UE_FeICIC_RR,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,'RR',0,'CCP',ideal);
        if ~ideal
            for ue = 1:NumUE
                [UE_FeICIC_RR(ue)] = Throughput(UE_FeICIC_RR(ue).attach,UE_FeICIC_RR(ue),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
            end
        end
        TimePerIt_FeICIC_RR(it) = toc(timer_FeICIC_RR);
        disp('FeICIC optimization using CCP and RR done')
        
        timer_FeICIC_PF_LTE = tic;
        [Station_FeICIC_PF_LTE,UE_FeICIC_PF_LTE] = BetterResp_FeICIC...
            (Station_FeICIC_PF_LTE,ix,UE_FeICIC_PF_LTE,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,'PF_LTE',10,'CCP',ideal);
        if ~ideal
            for ue = 1:NumUE
                [UE_FeICIC_PF_LTE(ue)] = Throughput(UE_FeICIC_PF_LTE(ue).attach,UE_FeICIC_PF_LTE(ue),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
            end
        end
        TimePerIt_FeICIC_PF_LTE(it) = toc(timer_FeICIC_PF_LTE);
        disp('FeICIC optimization using CCP and PF LTE done')
        
        timer_FeICIC_CC = tic;
        [Station_FeICIC_CC,UE_FeICIC_CC] = BetterResp_FeICIC...
            (Station_FeICIC_CC,ix,UE_FeICIC_CC,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,'CC',10,'CCP',ideal);
        if ~ideal
            for ue = 1:NumUE
                [UE_FeICIC_CC(ue)] = Throughput(UE_FeICIC_CC(ue).attach,UE_FeICIC_CC(ue),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
            end
        end
        TimePerIt_FeICIC_CC(it) = toc(timer_FeICIC_CC);
        disp('FeICIC optimization using CCP and cake-cutting done')
        
        timer_eICIC_RR = tic;
        [Station_eICIC_RR,UE_eICIC_RR] = BestResp_eICIC(Station_eICIC_RR,ix,UE_eICIC_RR,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,'RR',0);
        if ~ideal
            for ue = 1:NumUE
                [UE_eICIC_RR(ue)] = Throughput(UE_eICIC_RR(ue).attach,UE_eICIC_RR(ue),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
            end
        end
        TimePerIt_eICIC_RR(it) = toc(timer_eICIC_RR);
        disp('eICIC optimization using RR done')
        
        timer_eICIC_PF_LTE = tic;
        [Station_eICIC_PF_LTE,UE_eICIC_PF_LTE] = BestResp_eICIC(Station_eICIC_PF_LTE,ix,UE_eICIC_PF_LTE,CSB_value,NumTimeSlots,NumFreqSlots,NumUE,NoiseDen,NoiseFigure,BWperRB,ABSset,NumABS,'PF_LTE',10);
        if ~ideal
            for ue = 1:NumUE
                [UE_eICIC_PF_LTE(ue)] = Throughput(UE_eICIC_PF_LTE(ue).attach,UE_eICIC_PF_LTE(ue),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
            end
        end
        TimePerIt_eICIC_PF_LTE(it) = toc(timer_eICIC_PF_LTE);
        disp('eICIC optimization using PF LTE done')
               
        %% Obtain the global utility
        GU_FeICIC_RR(it,topology) = sum(log([UE_FeICIC_RR.throughput]));
        GU_eICIC_RR(it,topology) = sum(log([UE_eICIC_RR.throughput]));
        
        GU_FeICIC_PF_LTE(it,topology) = sum(log([UE_FeICIC_PF_LTE.throughput]));
        GU_eICIC_PF_LTE(it,topology) = sum(log([UE_eICIC_PF_LTE.throughput]));
        
        GU_FeICIC_CC(it,topology) = sum(log([UE_FeICIC_CC.throughput]));
        
        if ideal == 1
            if GU_FeICIC_RR(it,topology) < GU_FeICIC_RR(it-1,topology) || ...
                    GU_FeICIC_PF_LTE(it,topology) < GU_FeICIC_PF_LTE(it-1,topology) || ...
                    GU_FeICIC_CC(it,topology) < GU_FeICIC_CC(it-1,topology) || ...
                    GU_eICIC_RR(it,topology) < GU_eICIC_RR(it-1,topology) || ...
                    GU_eICIC_PF_LTE(it,topology) < GU_eICIC_PF_LTE(it-1,topology)
                
                error('Utility decreases!!!')
            end
        end
    end
    %% Record the results
    NumCenterUE = length(UE_ix_center);
    
    AvgTimePerIt_FeICIC_CC(topology) = mean(TimePerIt_FeICIC_CC(2:it));
    AvgTimePerIt_FeICIC_PF_LTE(topology) = mean(TimePerIt_FeICIC_PF_LTE(2:it));
    AvgTimePerIt_FeICIC_RR(topology) = mean(TimePerIt_FeICIC_RR(2:it));
    
    AvgTimePerIt_eICIC_PF_LTE(topology) = mean(TimePerIt_eICIC_PF_LTE(2:it));
    AvgTimePerIt_eICIC_RR(topology) = mean(TimePerIt_eICIC_RR(2:it));
    
    rate_RR((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_RR(UE_ix_center).throughput;
    energy_RR(topology) = FindEnergy(Station_RR,UE_RR(UE_ix_center),0.001);
    cell_edge_rate_RR_this_topology = sort( [ UE_RR(UE_ix_center).throughput ] );
    cell_edge_rate_RR_this_topology = cell_edge_rate_RR_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_RR(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)...
        = cell_edge_rate_RR_this_topology;
    
    rate_PF_LTE((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_PF_LTE(UE_ix_center).throughput;
    energy_PF_LTE(topology) = FindEnergy(Station_PF_LTE,UE_PF_LTE(UE_ix_center),0.001);
    cell_edge_rate_PF_LTE_this_topology = sort( [ UE_PF_LTE(UE_ix_center).throughput ] );
    cell_edge_rate_PF_LTE_this_topology = cell_edge_rate_PF_LTE_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_PF_LTE(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)...
        = cell_edge_rate_PF_LTE_this_topology;
    
    rate_CC((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_CC(UE_ix_center).throughput;
    energy_CC(topology) = FindEnergy(Station_CC,UE_CC(UE_ix_center),0.001);
    cell_edge_rate_CC_this_topology = sort( [ UE_CC(UE_ix_center).throughput ] );
    cell_edge_rate_CC_this_topology = cell_edge_rate_CC_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_CC(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)...
        = cell_edge_rate_CC_this_topology;
    
    rate_eICIC_RR((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_eICIC_RR(UE_ix_center).throughput;
    energy_eICIC_RR(topology) = FindEnergy(Station_eICIC_RR,UE_eICIC_RR(UE_ix_center),0.001);
    cell_edge_rate_eICIC_RR_this_topology = sort( [ UE_eICIC_RR(UE_ix_center).throughput ] );
    cell_edge_rate_eICIC_RR_this_topology = cell_edge_rate_eICIC_RR_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_eICIC_RR(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)... 
        = cell_edge_rate_eICIC_RR_this_topology;
    
    rate_eICIC_PF_LTE((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_eICIC_PF_LTE(UE_ix_center).throughput;
    energy_eICIC_PF_LTE(topology) = FindEnergy(Station_eICIC_PF_LTE,UE_eICIC_PF_LTE(UE_ix_center),0.001);
    cell_edge_rate_eICIC_PF_LTE_this_topology = sort( [ UE_eICIC_PF_LTE(UE_ix_center).throughput ] );
    cell_edge_rate_eICIC_PF_LTE_this_topology = cell_edge_rate_eICIC_PF_LTE_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_eICIC_PF_LTE(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)... 
        = cell_edge_rate_eICIC_PF_LTE_this_topology;
    
    rate_FeICIC_RR((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_FeICIC_RR(UE_ix_center).throughput;
    energy_FeICIC_RR(topology) = FindEnergy(Station_FeICIC_RR,UE_FeICIC_RR(UE_ix_center),0.001);
    cell_edge_rate_FeICIC_RR_this_topology = sort( [ UE_FeICIC_RR(UE_ix_center).throughput ] );
    cell_edge_rate_FeICIC_RR_this_topology = cell_edge_rate_FeICIC_RR_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_FeICIC_RR(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)... 
        = cell_edge_rate_FeICIC_RR_this_topology;
    
    rate_FeICIC_PF_LTE((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_FeICIC_PF_LTE(UE_ix_center).throughput;
    energy_FeICIC_PF_LTE(topology) = FindEnergy(Station_FeICIC_PF_LTE,UE_FeICIC_PF_LTE(UE_ix_center),0.001);
    cell_edge_rate_FeICIC_PF_LTE_this_topology = sort( [ UE_FeICIC_PF_LTE(UE_ix_center).throughput ] );
    cell_edge_rate_FeICIC_PF_LTE_this_topology = cell_edge_rate_FeICIC_PF_LTE_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_FeICIC_PF_LTE(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)... 
        = cell_edge_rate_FeICIC_PF_LTE_this_topology;
    
    rate_FeICIC_CC((topology-1)*NumCenterUE+1:topology*NumCenterUE) = UE_FeICIC_CC(UE_ix_center).throughput;
    energy_FeICIC_CC(topology) = FindEnergy(Station_FeICIC_CC,UE_FeICIC_CC(UE_ix_center),0.001);
    cell_edge_rate_FeICIC_CC_this_topology = sort( [ UE_FeICIC_CC(UE_ix_center).throughput ] );
    cell_edge_rate_FeICIC_CC_this_topology = cell_edge_rate_FeICIC_CC_this_topology(1:NumCellEdgeUE);
    cell_edge_rate_FeICIC_CC(cell_edge_count:cell_edge_count+NumCellEdgeUE-1)... 
        = cell_edge_rate_FeICIC_CC_this_topology;
    
    cell_edge_count = cell_edge_count + NumCellEdgeUE;
    
    if topology == Topology
        cell_edge_rate_RR(cell_edge_count:end) = [];
        cell_edge_rate_PF_LTE(cell_edge_count:end) = [];
        cell_edge_rate_CC(cell_edge_count:end) = [];
        cell_edge_rate_eICIC_RR(cell_edge_count:end) = [];
        cell_edge_rate_eICIC_PF_LTE(cell_edge_count:end) = [];
        cell_edge_rate_FeICIC_RR(cell_edge_count:end) = [];
        cell_edge_rate_FeICIC_PF_LTE(cell_edge_count:end) = [];
        cell_edge_rate_FeICIC_CC(cell_edge_count:end) = [];
    end
     
    save (FileName,'Performed_iteration','MaxIteration','ideal','NumPicoPerCell',...
        'NumUEPerCell','NumTimeSlots','NumFreqSlots','UE_near_Pico_Fraction',...
        'ABSset','CSB_value','BWperRB',...
        'AvgTimePerIt_FeICIC_PF_LTE','AvgTimePerIt_FeICIC_RR',...
        'AvgTimePerIt_eICIC_PF_LTE','AvgTimePerIt_eICIC_RR',...
        ...
        'GU_FeICIC_RR','GU_FeICIC_PF_LTE','GU_FeICIC_CC','GU_eICIC_RR','GU_eICIC_PF_LTE',...
        ...
        'rate_RR', 'rate_PF_LTE', 'rate_CC',...
        'rate_eICIC_RR', 'rate_eICIC_PF_LTE',...
        'rate_FeICIC_RR',  'rate_FeICIC_PF_LTE', 'rate_FeICIC_CC',...
        ...
        'energy_RR', 'energy_PF_LTE', 'energy_CC',...
        'energy_eICIC_RR', 'energy_eICIC_PF_LTE',...
        'energy_FeICIC_RR',  'energy_FeICIC_PF_LTE', 'energy_FeICIC_CC',...
        ...
        'cell_edge_rate_RR', 'cell_edge_rate_PF_LTE', 'cell_edge_rate_CC',...
        'cell_edge_rate_eICIC_RR', 'cell_edge_rate_eICIC_PF_LTE',...
        'cell_edge_rate_FeICIC_RR', 'cell_edge_rate_FeICIC_PF_LTE', 'cell_edge_rate_FeICIC_CC'...
        ); 
        
end

end_time = datestr(clock)

%% Summarize the results
% For energy efficiency
EE_RR = sum(rate_RR) * 0.001 * BWperRB / sum(energy_RR);
EE_PF_LTE = sum(rate_PF_LTE) * 0.001 * BWperRB / sum(energy_PF_LTE);
EE_CC = sum(rate_CC) * 0.001 * BWperRB / sum(energy_CC);
EE_eICIC_RR = sum(rate_eICIC_RR) * 0.001 * BWperRB / sum(energy_eICIC_RR);
EE_eICIC_PF_LTE = sum(rate_eICIC_PF_LTE) * 0.001 * BWperRB / sum(energy_eICIC_PF_LTE);
EE_FeICIC_RR = sum(rate_FeICIC_RR) * 0.001 * BWperRB / sum(energy_FeICIC_RR);
EE_FeICIC_PF_LTE = sum(rate_FeICIC_PF_LTE) * 0.001 * BWperRB / sum(energy_FeICIC_PF_LTE);
EE_FeICIC_CC = sum(rate_FeICIC_CC) * 0.001 * BWperRB / sum(energy_FeICIC_CC);

% For average throughput (spectral efficiency)
SE_RR = sum(rate_RR) / length(rate_RR);
SE_PF_LTE = sum(rate_PF_LTE) / length(rate_PF_LTE);
SE_CC = sum(rate_CC) / length(rate_CC);
SE_eICIC_RR = sum(rate_eICIC_RR) / length(rate_eICIC_RR);
SE_eICIC_PF_LTE = sum(rate_eICIC_PF_LTE) / length(rate_eICIC_PF_LTE);
SE_FeICIC_RR = sum(rate_FeICIC_RR) / length(rate_FeICIC_RR);
SE_FeICIC_PF_LTE = sum(rate_FeICIC_PF_LTE) / length(rate_FeICIC_PF_LTE);
SE_FeICIC_CC = sum(rate_FeICIC_CC) / length(rate_FeICIC_CC);

% For Jain's fairness index
Jain_RR = jain(rate_RR);
Jain_PF_LTE = jain(rate_PF_LTE);
Jain_CC = jain(rate_CC);
Jain_eICIC_RR = jain(rate_eICIC_RR);
Jain_eICIC_PF_LTE = jain(rate_eICIC_PF_LTE);
Jain_FeICIC_RR = jain(rate_FeICIC_RR);
Jain_FeICIC_PF_LTE = jain(rate_FeICIC_PF_LTE);
Jain_FeICIC_CC = jain(rate_FeICIC_CC);

% Summary of above results in table
rowname = {'RR';'PF_LTE';'CC';'eICIC RR';'eICIC PF_LTE';'FeICIC RR';'FeICIC PF_LTE';'FeICIC CC'};
EE_bits_per_Joule = [EE_RR;EE_PF_LTE;EE_CC;EE_eICIC_RR;EE_eICIC_PF_LTE;EE_FeICIC_RR;EE_FeICIC_PF_LTE;EE_FeICIC_CC];
SE_bps_per_Hz = [SE_RR;SE_PF_LTE;SE_CC;SE_eICIC_RR;SE_eICIC_PF_LTE;SE_FeICIC_RR;SE_FeICIC_PF_LTE;SE_FeICIC_CC];
Jain = [Jain_RR;Jain_PF_LTE;Jain_CC;Jain_eICIC_RR;Jain_eICIC_PF_LTE;Jain_FeICIC_RR;Jain_FeICIC_PF_LTE;Jain_FeICIC_CC];
table(EE_bits_per_Joule,SE_bps_per_Hz,Jain,'RowNames',rowname)

% For cell-edge throughput CDF
figure
h = cdfplot(cell_edge_rate_PF_LTE);
set(h,'LineStyle',':','color','k')
hold on
h = cdfplot(cell_edge_rate_eICIC_PF_LTE);
set(h,'LineStyle','--','color','r')
h = cdfplot(cell_edge_rate_FeICIC_PF_LTE);
set(h,'LineStyle','-','color','b')
hold off
grid on
xlabel('Achieved user rate (bps/Hz)')
ylabel('CDF of worst 5% UEs'' achieved rates')
legend('Nil, PF LTE','eICIC, PF LTE','FeICIC, PF LTE','Location','northwest')