%% Generate a random network topology

%% The interference range of a base station is defined as the union of N(m_u,0), N(m_u,1)

% Use wrap-around model: Create a 3-layer haxagonal layout (19 cells), 
% then copy the layout 6 times and place the copies around the layout.

% In the comments, eNB stands for macro base station (BS)

function [NumUE,NoiseDen,NoiseFigure,BWperRB,NumeNB,Station,eNB,Pico,UE]...
 = wrap_around_model_random(NumeNBPerCluster,NumPicoPerCell,NumUEPerCell,NumTimeSlots,NumFreqSlots,UE_near_Pico_Fraction)

rng('shuffle')

% The interference and association ranges are as follows:
% For interference: Stations located within one tiers from the location of 
% a user contribute to the interference power
% For association: Users can only be associated to stations within one tier
% of their locations
Layers = 3; % In case we have 37 eNBs, there are 4 layers from the center to the outer layer.
            % Other values of Layers are not tested and may lead to error
InterfereRange = 2; % Layers of eNBs that are interfered by an eNB including itself

% Number of eNBs, assume symmetric haxagonal layout
NumeNB = NumeNBPerCluster * 7; % Total eNBs in the network

% Number of eNBs interfered by an eNB including itself
NumeNBAffected = FindNumHexagon(InterfereRange); 

NumPico = NumPicoPerCell * NumeNB;
NumUE = NumUEPerCell * FindNumHexagon(Layers+2);
NumPRB = NumTimeSlots*NumFreqSlots;

NeighborCellDistance = 1000; % Give the distance between two neighbor eNBs in meters
GuardDisteNBToUE = 35; % The minimum distance from a UE to an eNB in meters
GuardDistPicoToUE = 10; % The minimum distance from a UE to a Pico in meters
GuardDistPicoToPico = 40; % The minimum distance from a Pico to another Pico in meters
GuardDistPicoToeNB = 75; % The minimum distance from a Pico to an eNB in meters

MaxPowereNB = 40; % Maximum power of an eNB in watt
MaxPowerPico = 1; % Maximum power of a Pico in watt

NoiseDen = -174; % Noise density in dBm/Hz
NoiseFigure = 9; % Noise figure in dB
BWperRB = 180 * 1000; % Bandwidth per resource block in Hz


eNB(NumeNB,1) = struct('Type',[],'NameTag',[],'MacroID',[],'ID',[],'Cluster',[],...
    'X',[],'Y',[],'Boundary',[],'Neighbor1stTier',[],'Neighbor2ndTier',[],...
    'AllNeighbor',[],'Interfering',[],'CandUE',[],'UEinPRB',[],'PRBpower',[],'ABS',[],'CSB',[]);
% The structure for eNB:
% 'Type': Type of this station. 1 means eNB, 2 means Pico
% 'NameTag': the name of the eNB, e.g. 3.1, indicating that the eNB is the 3rd eNB in cluster 1
% 'MacroID': Meaningless here
% 'ID': ID of each eNB, duplicated across clusters
% 'Cluster': shows which one of the 7 clusters does the eNB belong to
% 'X': the x-axis coordinate of the position of an eNB
% 'Y': the y-axis coordinate of the position of an eNB
% 'Boundary': The coordinates of the 6 boundary points of a cell, Boundary(k,1) and Boundary(k,2) will be the x and y coordinate of the kth point, respectively.
% 'Neighbor1stTier': Record the 1-hop neighbor cells' unique identification, in ID.Cluster form (i.e., 3.1 for the 3rd cell in cluster 1)
% 'Neighbor2ndTier': Record the 2-hop neighbor cell's unique identification, in ID.Cluster form
% 'AllNeighbor': All neighbours of the cell
% 'Interfering': The list of UEs who are interfered by this station
% 'CandUE': List of UEs who may be attached to this station
% 'UEinPRB': indicate which user (in terms of user ID) is allocated to a
%            particular PRB (physical resource block), it is arranged as 
%            follows: [SubframesOn1stRB SubframesOn2ndRB ...], where
%            subframe is on time domain and RB is on frequency domain
% 'PRBpower': the transmission power of each eNB at each PRB, which is
%             equal to the max power divided by number of RBs (frequency slots),
%             similarly arranged as UEinPRB
% 'ABS': tells which time slots the eNB will keep silence, [1 0 1] means
%        keep slience at the second time slots on a RB. Similarly arranged as 
%        UEinPRB. In current model, ABS patterns on all RBs are the same
% 'CSB': Cell selection bias value in dB

if NumPico > 0
Pico(NumPico,1) = struct('Type',[],'NameTag',[],'MacroID',[],'ID',[],'Cluster',[],...
    'X',[],'Y',[],'Boundary',[],'Neighbor1stTier',[],'Neighbor2ndTier',[],'AllNeighbor',[],...
    'Interfering',[],'CandUE',[],'UEinPRB',[],'PRBpower',[],'ABS',[],'CSB',[]);
else
    Pico = [];
end
% 'NameTag': the name of the Pico, e.g. 2003.1, indicating that the Pico is the 2nd Pico in the 3rd eNB in cluster 1
% 'Cluster': Record which cluster this Pico belongs to
% 'MacroID': Record which Macro cell this Pico belongs to
% The other structures of Pico is similar to eNB except it has no ABS
% Note that the IDs of Picos are duplicated across clusters, but not Macro cells in the same cluster

UE(NumUE,1) = struct('ID',[],'X',[],'Y',[],'MacroID',[],'Cluster',[],...
    'Interfere',[],'InterfereMat',[],'InterfereIx',[],'InterferePower',[],'Shadow',[],...
    'PLandSw',[],'FastFading',[],'ABS_Interfere',[],'RSS',[],'HostCand',[],'PilotRSS',[],...
    'CSB_HostCand',[],'attach',[],'weight',[],'Allocation',[],'Allocation_decimal',[],...
    'ThroughputPerPRB',[],'throughput',[],'throughput_fractional_allocation',[],'SINR_PerPRB_FullPower',[]);
% The structure for UE:
% 'ID', 'X', 'Y' are the same as the eNB
% 'MacroID': tells which marco cell this UE belongs to
% 'Cluster': tells which cluster this UE belongs to
% 'Interfere': lists the eNBs and Picos, within two tiers, that can interfere the UE:
%              eNB that interferes will be stored as MacroID+Cluster/10;
%              Pico that interferes will be stored as PicoID*100+MacroID+Cluster/10;
%              Will store an eNB followed by Picos in it, and then another eNB
% 'InterfereMat': Replications of Interfere
% 'InterfereIx': The index of each interferer in the structure [eNB;Pico]
% 'InterferePower': The power of the corresponding interferer in Watt
% 'Shadow': the log-normal shadowing variable from eNB and Pico in dB, consider eNB and Pico within two tiers
% 'PLandSw': Path loss and shadowing in dB from the corresponding interferers
% 'FastFading': Fast fading channel gains from all interferers on each PRB
% 'ABS_Interfere': ABS pattern of the corresponding intereferers
% 'RSS': Receive signal strength from all interferers
% 'HostCand': lists the potential Macro and Pico that the UE may attach to, consider eNBs and Picos within one tier
%             The same list will be in the beginning of Interfere  
% 'PilotRSS': the receive signal strength of the pilot signals from eNB and Pico from HostCand, in watt
% 'CSB_HostCand': the CSB value of the corrsponding host candidates in dB
% 'attach': tells the ID of the eNB or Pico that a UE is attached to
% 'weight': Weight of the UE
% 'Allocation': Tells the allocation of RBs of the UE
% 'Allocation_decimal': Record the allocation pattern by PF scheduler
%                       before rounding
% 'ThroughputPerPRB': gives the throughput of this UE in each PRB (after attachment), it is useful for the allocation optimization
%                    -1 means that the RB is an ABS 
% 'throughput': gives the throughput of a UE
% 'throughput_fractional_allocation': gives the throughput of a UE
%                                     according to Allocation_decimal
% 'SINR_PerPRB_FullPower': gives the SINR on each PRB assuming all nearby
%                          BSs are transmitting at full power

% Initialize the structures
for j = 1:7
    for i = 1:NumeNBPerCluster
        eNB((j-1)*NumeNBPerCluster+i).Type = 1;
        eNB((j-1)*NumeNBPerCluster+i).NameTag = i+j/10;
        eNB((j-1)*NumeNBPerCluster+i).ID = i;
        eNB((j-1)*NumeNBPerCluster+i).Cluster = j;
        eNB((j-1)*NumeNBPerCluster+i).X = 0;
        eNB((j-1)*NumeNBPerCluster+i).Y = 0;
        eNB((j-1)*NumeNBPerCluster+i).Boundary = zeros(6,2); % The east point will be stored first, then the north-east, ...
        eNB((j-1)*NumeNBPerCluster+i).Neighbor1stTier = zeros(6,1);
        eNB((j-1)*NumeNBPerCluster+i).Neighbor2ndTier = zeros(12,1);
        eNB((j-1)*NumeNBPerCluster+i).AllNeighbor = [];
        eNB((j-1)*NumeNBPerCluster+i).Interfering = zeros(NumUEPerCell*NumeNBAffected,1);
        eNB((j-1)*NumeNBPerCluster+i).CandUE = [];
        eNB((j-1)*NumeNBPerCluster+i).UEinPRB = zeros(1,NumPRB);
        eNB((j-1)*NumeNBPerCluster+i).PRBpower = MaxPowereNB/NumFreqSlots*ones(1,NumPRB);
        eNB((j-1)*NumeNBPerCluster+i).ABS = ones(1,NumPRB);
        eNB((j-1)*NumeNBPerCluster+i).CSB = 0;
    end
end

for i = 1:7
    for j = 1:NumeNBPerCluster
        for k = 1:NumPicoPerCell
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).Type = 2;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).NameTag = k*1000+j+i/10;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).Cluster = i;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).MacroID = j;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).ID = k;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).X = 0;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).Y = 0;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).AllNeighbor = [];
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).Interfering = zeros(NumUEPerCell*NumeNBAffected,1);
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).CandUE = [];
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).UEinPRB = zeros(1,NumPRB);
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).PRBpower = MaxPowerPico/NumFreqSlots*ones(1,NumPRB);
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).ABS = ones(1,NumPRB);
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).CSB = 0;
        end
    end
end

for i = 1:NumUE
    UE(i).ID = i;
    UE(i).X = 0;
    UE(i).Y = 0;
    if i <= NumUEPerCell * NumeNBPerCluster
        UE(i).MacroID = ceil(i/NumUEPerCell);
        UE(i).Cluster = 1;
    else
        UE(i).MacroID = 0;
        UE(i).Cluster = 0;
    end
    UE(i).Interfere = zeros(7*(NumPicoPerCell+1),1);
    UE(i).InterfereMat = zeros(7*(NumPicoPerCell+1),NumPRB);
    UE(i).InterfereIx = zeros(7*(NumPicoPerCell+1),1);
    UE(i).InterferePower = zeros(7*(NumPicoPerCell+1),NumPRB);
    UE(i).Shadow = zeros(7*(NumPicoPerCell+1),1);
    UE(i).PLandSw = zeros(7*(NumPicoPerCell+1),NumPRB);
    UE(i).FastFading = sqrt(randn(7*(NumPicoPerCell+1),NumPRB).^2./2 + randn(7*(NumPicoPerCell+1),NumPRB).^2./2); 
    UE(i).ABS_Interfere = ones(7*(NumPicoPerCell+1),NumPRB);
    UE(i).RSS = zeros(7*(NumPicoPerCell+1),NumPRB);
    % Consider attaching to eNBs and Picos within one tier
    UE(i).HostCand = zeros(7+7*NumPicoPerCell,1);
    UE(i).PilotRSS = zeros(7+7*NumPicoPerCell,1);
    UE(i).CSB_HostCand = zeros(7+7*NumPicoPerCell,1);
    UE(i).attach = 0;
    UE(i).weight = 1;
    UE(i).Allocation = zeros(1,NumPRB);
    UE(i).Allocation_decimal = zeros(1,NumPRB);
    UE(i).ThroughputPerPRB = zeros(1,NumPRB);
    UE(i).throughput = 0;  
    UE(i).throughput_fractional_allocation = 0;
    UE(i).SINR_PerPRB_FullPower = zeros(1,NumPRB);
end

% Find the locations of the cells
for i = 1:7
    if i == 1 % The center cluster
        X_center = 0;
        Y_center = 0;
        eNBIx = 1;
        eNB = CreateCell(eNB,X_center,Y_center,Layers,eNBIx,NeighborCellDistance,NumeNBPerCluster);
    elseif i == 2 % The cluster with center just above x-axis and to the right of y-axis
        Y_center = NeighborCellDistance / 2 + Y_center;
        X_center = (NeighborCellDistance/2*2/sqrt(3)+NeighborCellDistance/2/sqrt(3)) * (2*Layers-1) + X_center;
        dist = sqrt((X_center-eNB(1).X)^2 + (Y_center-eNB(1).Y)^2);
        angle = atan(abs(Y_center-eNB(1).Y)/abs(X_center-eNB(1).X));
        eNBIx = 1 + NumeNBPerCluster;
        eNB = CreateCell(eNB,X_center,Y_center,Layers,eNBIx,NeighborCellDistance,NumeNBPerCluster);
    else % The other clusters
        angle = (angle / 2 / pi * 360 + 60) / 360 * 2 * pi;
        X_center = eNB(1).X + dist * cos(angle);
        Y_center = eNB(1).Y + dist * sin(angle);
        eNBIx = 1 + (i-1)*NumeNBPerCluster;
        eNB = CreateCell(eNB,X_center,Y_center,Layers,eNBIx,NeighborCellDistance,NumeNBPerCluster);
    end
end
% Find each cell's neighbors
for i = 1:NumeNB
    ix1 = 0;
    ix2 = 0;
    for j = 1:NumeNB
        if abs(sqrt((eNB(i).X-eNB(j).X)^2 + (eNB(i).Y-eNB(j).Y)^2) - NeighborCellDistance) < 0.0001
            ix1 = ix1 + 1;
            eNB(i).Neighbor1stTier(ix1) = eNB(j).ID + eNB(j).Cluster/10;
        elseif sqrt((eNB(i).X-eNB(j).X)^2 + (eNB(i).Y-eNB(j).Y)^2) > NeighborCellDistance*1.1 && sqrt((eNB(i).X-eNB(j).X)^2 + (eNB(i).Y-eNB(j).Y)^2) < NeighborCellDistance*2.1
            ix2 = ix2 + 1;
            eNB(i).Neighbor2ndTier(ix2) = eNB(j).ID + eNB(j).Cluster/10;
        end
    end
end

% First set which cell does each Pico belong to
for i = 1:7
    for j = 1:NumeNBPerCluster
        for k = 1:NumPicoPerCell
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).Cluster = i;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).MacroID = j;
            Pico((i-1)*NumeNBPerCluster*NumPicoPerCell+(j-1)*NumPicoPerCell+k).ID = k;
        end
    end
end            
% Put Picos randomly into each big cell
for i = 1:NumeNBPerCluster
    for j = 1:NumPicoPerCell  
        good = -1;
        while good < NumeNB*NumPicoPerCell - 1
            good = 0;
            [Pico((i-1)*NumPicoPerCell+j).X,Pico((i-1)*NumPicoPerCell+j).Y] = ...
                GenPicoPositionRandom(eNB(i).X,eNB(i).Y,NeighborCellDistance,GuardDistPicoToeNB);
            % Check distance constraint between Picos
            for k = 1:NumeNB*NumPicoPerCell
                if (Pico(k).X-Pico((i-1)*NumPicoPerCell+j).X)^2 + (Pico(k).Y-Pico((i-1)*NumPicoPerCell+j).Y)^2 ...
                        >= GuardDistPicoToPico^2 && k ~= (i-1)*NumPicoPerCell+j
                    good = good + 1;
                end
            end
        end
        % Copy the position of this Pico to other clusters
        for k = 2:7
            Pico((k-1)*NumeNBPerCluster*NumPicoPerCell+(i-1)*NumPicoPerCell+j).X = ...
                Pico((i-1)*NumPicoPerCell+j).X + eNB((k-1)*NumeNBPerCluster+1).X;
            Pico((k-1)*NumeNBPerCluster*NumPicoPerCell+(i-1)*NumPicoPerCell+j).Y = ...
                Pico((i-1)*NumPicoPerCell+j).Y + eNB((k-1)*NumeNBPerCluster+1).Y;
        end         
    end
end

% Find all neighbours of each station within two tiers
for i = 1:NumeNB
    temp = [eNB(i).Neighbor1stTier(find(eNB(i).Neighbor1stTier));eNB(i).Neighbor2ndTier(find(eNB(i).Neighbor2ndTier))];
    eNB(i).AllNeighbor = temp;
    for k = 1:NumPicoPerCell
        eNB(i).AllNeighbor = [eNB(i).AllNeighbor;eNB(i).NameTag+k*1000];
    end
    for j = 1:length(temp)
        for k = 1:NumPicoPerCell
            eNB(i).AllNeighbor = [eNB(i).AllNeighbor;eNB(i).AllNeighbor(j)+k*1000];
        end
    end
end
for i = 1:NumPico
    eNBix = find([eNB.NameTag]==Pico(i).MacroID+Pico(i).Cluster/10);
    Pico(i).AllNeighbor = eNB(eNBix).AllNeighbor;
    Pico(i).AllNeighbor(find([Pico(i).AllNeighbor]==Pico(i).NameTag)) = [];
    Pico(i).AllNeighbor = [eNB(eNBix).NameTag;Pico(i).AllNeighbor];
end

% For the first cluster, randomly generate the positions of UEs.
% For each cell, (UE_near_Pico_Fraction)% of UEs will be randomly placed in 
% circles whose centers are the picos. The other users will be randomly 
% placed inside the haxagon, including the areas around Picos
for i = 1:NumeNBPerCluster
    UEcount = 0;
    ThisUEMacroIx = ReturnMacroIx(i, 1, NumeNBPerCluster);
    % Put (UE_near_Pico_Fraction)% of UEs near the Picos
    %%{
    for j = 1:NumPicoPerCell
        while UEcount < NumUEPerCell/NumPicoPerCell * j * UE_near_Pico_Fraction
            UEcount = UEcount + 1;
            good = 0;
            while good < 7 * NumPicoPerCell + 1
                good = 0;
                % First generate a random angle, measured from 3 o'clock counter-clockwisely
                angle = 2*pi*rand;
                % Then randomly generate a distance from the center of the Pico 
                d = (GuardDistPicoToPico*2.5-GuardDistPicoToUE)*rand + GuardDistPicoToUE;
                % Find the coordinates of the UE
                UE((i-1)*NumUEPerCell+UEcount).X = Pico((i-1)*NumPicoPerCell+j).X + d * cos(angle);
                UE((i-1)*NumUEPerCell+UEcount).Y = Pico((i-1)*NumPicoPerCell+j).Y + d * sin(angle);
                % Check whether the UE gets too close to a Pico in a neighbor eNB or a Pico in its own eNB
                for m = 1:7
                    if m < 7 % If it is checking Picos from neighbor eNBs
                        NeighborMacroIx = ReturnMacroIx(floor(eNB(ThisUEMacroIx).Neighbor1stTier(m)), ...
                            eNB(ThisUEMacroIx).Neighbor1stTier(m)*10-floor(eNB(ThisUEMacroIx).Neighbor1stTier(m))*10, ...
                            NumeNBPerCluster);
                        for n = 1:NumPicoPerCell
                            NeighborPicoIx = ReturnPicoIx(n, eNB(NeighborMacroIx).ID, ...
                                eNB(NeighborMacroIx).Cluster, NumPicoPerCell, NumeNBPerCluster);
                            if (UE((i-1)*NumUEPerCell+UEcount).X-Pico(NeighborPicoIx).X)^2 ...
                                    + (UE((i-1)*NumUEPerCell+UEcount).Y-Pico(NeighborPicoIx).Y)^2 ...
                                    >= GuardDistPicoToUE^2
                                good = good + 1;
                            end
                        end
                    else % If it is checking Picos from its own eNB
                        % Check whether the UE is too close to the eNB 
                        if (UE((i-1)*NumUEPerCell+UEcount).X-eNB(ThisUEMacroIx).X)^2 ...
                                + (UE((i-1)*NumUEPerCell+UEcount).Y-eNB(ThisUEMacroIx).Y)^2 ...
                                >= GuardDisteNBToUE^2
                            good = good + 1;
                        end
                        for n = 1:NumPicoPerCell
                            PicoIx = ReturnPicoIx(n,i,1,NumPicoPerCell,NumeNBPerCluster);
                            if (UE((i-1)*NumUEPerCell+UEcount).X-Pico(PicoIx).X)^2 ...
                                    + (UE((i-1)*NumUEPerCell+UEcount).Y-Pico(PicoIx).Y)^2 ...
                                    >= GuardDistPicoToUE^2
                                good = good + 1;
                            end
                        end
                    end
                end
            end
        end
    end
    %}
    % Put the other UEs in the haxagon
    while UEcount < NumUEPerCell
        UEcount = UEcount + 1;
        good = 0; % check whether the positions of UEs is too close to Picos
        if NumPicoPerCell > 0 % If there are Picos inside an eNB
            while good < NumPico
                good = 0;
                % Generate a random angle, measured from 3 o'clock counter-clockwisely
                angle = 2*pi*rand;
                % Find out the max distance allowed along that direction
                MaxDistAllowed = NeighborCellDistance/2 / sin(pi - (angle - floor(angle/pi*3)*pi/3) - pi/3);
                % Initialization
                d = rand * MaxDistAllowed;
                % Generate a random distance so that the UE will be located inside the
                % hexagon along that angle
                while d > MaxDistAllowed || d < GuardDisteNBToUE
                    d = MaxDistAllowed * rand;
                end
                
                % Find out the coordinates of the UE
                UE((i-1)*NumUEPerCell+UEcount).X = eNB(i).X + d * cos(angle);
                UE((i-1)*NumUEPerCell+UEcount).Y = eNB(i).Y + d * sin(angle);
                for k = 1:NumPico
                    if (UE((i-1)*NumUEPerCell+UEcount).X - Pico(k).X)^2 ...
                            + (UE((i-1)*NumUEPerCell+UEcount).Y - Pico(k).Y)^2 ...
                            >= GuardDistPicoToUE^2
                        good = good + 1;
                    end
                end
            end
        else % If there is no Pico inside an eNB
            % Generate a random angle, measured from 3 o'clock counter-clockwisely
            angle = 2*pi*rand;
            % Find out the max distance allowed along that direction
            MaxDistAllowed = NeighborCellDistance/2 / sin(pi - (angle - floor(angle/pi*3)*pi/3) - pi/3);
            % Initialization
            d = rand * MaxDistAllowed;
            % Generate a random distance so that the pico will be located inside the
            % hexagon along that angle
            while d > MaxDistAllowed || d < GuardDistPicoToeNB
                d = MaxDistAllowed * rand;
            end

            % Find out the coordinates of the UE
            UE((i-1)*NumUEPerCell+UEcount).X = eNB(i).X + d * cos(angle);
            UE((i-1)*NumUEPerCell+UEcount).Y = eNB(i).Y + d * sin(angle);
        end
    end
end
% Now copy the position of UEs from the first cluster to the other clusters, 
% with the constraint that only eNBs within the fifth layer counting from the 
% center eNB have UEs (i.e., 61 eNBs have UEs). This is because we only
% care about the utilities of the base stations in the center cluster
ix = NumUEPerCell * NumeNBPerCluster;
for i = 2:7
    for j = 1:NumeNBPerCluster
        MacroIx = ReturnMacroIx(j, i, NumeNBPerCluster);
        dist = sqrt((eNB(1).X-eNB(MacroIx).X)^2 + (eNB(1).Y-eNB(MacroIx).Y)^2); % Calculate the distance from current eNB to the center eNB
        if dist < 4 * NeighborCellDistance * 1.001
            for k = 1:NumUEPerCell
                ix = ix + 1;
                UE(ix).X = UE((j-1)*NumUEPerCell+k).X + eNB((i-1)*NumeNBPerCluster+1).X - eNB(1).X;
                UE(ix).Y = UE((j-1)*NumUEPerCell+k).Y + eNB((i-1)*NumeNBPerCluster+1).Y - eNB(1).Y;
                UE(ix).MacroID = j;
                UE(ix).Cluster = i;
            end
        end
    end
end
% Find out the Interfere, Shadow, HostCand, PilotRSS of each UE
CommonShadowing = exp(1.53806*randn); % To set correlations of shadowing, exp(1.53806*randn) is (roughly) lognormal distributed with SD 10
for i = 1:NumUE
    % The neighbors of an eNB are divided into two lists, so we just do it one by one
    ix = 0;
    % Find which Macro and Cluster this UE belongs to
    MacroIx = ReturnMacroIx(UE(i).MacroID,UE(i).Cluster,NumeNBPerCluster);
    MacroName = UE(i).MacroID + UE(i).Cluster/10;
    ix = ix + 1;
    UE(i).Interfere(ix) = MacroName;
    UE(i).InterfereMat(ix,:) = UE(i).Interfere(ix);
    UE(i).InterferePower(ix,:) = MaxPowereNB/NumFreqSlots*ones(1,NumPRB);
    % The mean and standard deviation is 0 and 10 dB, respectively.
    % Correlation is also considered, by adding a common shadowing and divide by sqrt(2)
    UE(i).Shadow(ix) = (exp(1.53806*randn) + CommonShadowing)/sqrt(2);
    UE(i).HostCand(ix) = MacroName;
    eNB(MacroIx).CandUE = [eNB(MacroIx).CandUE;i];

    PL = -( 128.1 + 37.6*log10( sqrt((UE(i).X-eNB(MacroIx).X)^2 + (UE(i).Y-eNB(MacroIx).Y)^2)/1000 ) + UE(i).Shadow(ix) ); % Path loss with shadowing in dB
    UE(i).PilotRSS(ix) = MaxPowereNB * 10^(PL/10);
    UE(i).PLandSw(ix,:) = 10*log10(UE(i).FastFading(ix,:)) + PL;
    UE(i).RSS(ix,:) = UE(i).InterferePower(ix,:).*10.^(UE(i).PLandSw(ix,:)/10);
    for j = 1:NumPicoPerCell
        ix = ix + 1;
        UE(i).Interfere(ix) = j*1000 + Pico((MacroIx-1)*NumPicoPerCell+j).MacroID + Pico((MacroIx-1)*NumPicoPerCell+j).Cluster/10;
        UE(i).InterfereMat(ix,:) = UE(i).Interfere(ix);
        UE(i).InterferePower(ix,:) = MaxPowerPico/NumFreqSlots*ones(1,NumPRB);
        UE(i).Shadow(ix) = (exp(1.53806*randn) + CommonShadowing)/sqrt(2);
        UE(i).HostCand(ix) = UE(i).Interfere(ix);
        PicoIx = ReturnPicoIx(j,Pico((MacroIx-1)*NumPicoPerCell+j).MacroID,Pico((MacroIx-1)*NumPicoPerCell+j).Cluster,NumPicoPerCell,NumeNBPerCluster);
        Pico(PicoIx).CandUE = [Pico(PicoIx).CandUE;i];
        
        PL = -( 140.7 + 36.7*log10( sqrt((UE(i).X-Pico(PicoIx).X)^2 + (UE(i).Y-Pico(PicoIx).Y)^2)/1000 ) + UE(i).Shadow(ix) ); % Path loss with shadowing in dB
        UE(i).PilotRSS(ix) = MaxPowerPico * 10^(PL/10);
        UE(i).PLandSw(ix,:) = 10*log10(UE(i).FastFading(ix,:)) + PL;
        UE(i).RSS(ix,:) = UE(i).InterferePower(ix,:).*10.^(UE(i).PLandSw(ix,:)/10);
    end
    % Then find the interferer in the first tier
    for j = 1:6
        if eNB(MacroIx).Neighbor1stTier(j) ~= 0
            ix = ix + 1;
            UE(i).Interfere(ix) = eNB(MacroIx).Neighbor1stTier(j);
            UE(i).InterfereMat(ix,:) = UE(i).Interfere(ix);
            UE(i).InterferePower(ix,:) = MaxPowereNB/NumFreqSlots*ones(1,NumPRB);
            UE(i).Shadow(ix) = (exp(1.53806*randn) + CommonShadowing)/sqrt(2);
            UE(i).HostCand(ix) = UE(i).Interfere(ix);
            MacroNeighborIx = ReturnMacroIx(floor(eNB(MacroIx).Neighbor1stTier(j)),int16(10*(eNB(MacroIx).Neighbor1stTier(j)-floor(eNB(MacroIx).Neighbor1stTier(j)))),NumeNBPerCluster );
            eNB(MacroNeighborIx).CandUE = [eNB(MacroNeighborIx).CandUE;i];
            
            PL = -( 128.1 + 37.6*log10( sqrt((UE(i).X-eNB(MacroNeighborIx).X)^2 + (UE(i).Y-eNB(MacroNeighborIx).Y)^2)/1000 ) + UE(i).Shadow(ix) ); % Path loss with shadowing in dB
            UE(i).PilotRSS(ix) = MaxPowerPico * 10^(PL/10);
            UE(i).PLandSw(ix,:) = 10*log10(UE(i).FastFading(ix,:)) + PL;
            UE(i).RSS(ix,:) = UE(i).InterferePower(ix,:).*10.^(UE(i).PLandSw(ix,:)/10);
            for k = 1:NumPicoPerCell
                ix = ix + 1;
                UE(i).Interfere(ix) = k*1000 + eNB(MacroIx).Neighbor1stTier(j);
                UE(i).InterfereMat(ix,:) = UE(i).Interfere(ix);
                UE(i).InterferePower(ix,:) = MaxPowerPico/NumFreqSlots*ones(1,NumPRB);
                UE(i).Shadow(ix) = (exp(1.53806*randn) + CommonShadowing)/sqrt(2);
                UE(i).HostCand(ix) = UE(i).Interfere(ix);
                PicoIx = ReturnPicoIx(k,floor(eNB(MacroIx).Neighbor1stTier(j)),int16(10*(eNB(MacroIx).Neighbor1stTier(j)-floor(eNB(MacroIx).Neighbor1stTier(j)))),NumPicoPerCell,NumeNBPerCluster);
                Pico(PicoIx).CandUE = [Pico(PicoIx).CandUE;i];
                
                PL = -( 140.7 + 36.7*log10( sqrt((UE(i).X-Pico(PicoIx).X)^2 + (UE(i).Y-Pico(PicoIx).Y)^2)/1000 ) + UE(i).Shadow(ix) ); % Path loss with shadowing in dB
                UE(i).PilotRSS(ix) = MaxPowerPico * 10^(PL/10);
                UE(i).PLandSw(ix,:) = 10*log10(UE(i).FastFading(ix,:)) + PL;
                UE(i).RSS(ix,:) = UE(i).InterferePower(ix,:).*10.^(UE(i).PLandSw(ix,:)/10);
            end
        end
    end
end

% Complete the list of UEs interfered by each station
for ue = 1:NumUE
    % Find out the UE's location in terms of eNB
    eNBname = UE(ue).MacroID+UE(ue).Cluster/10;
    eNBix = find([eNB.NameTag]==eNBname);
    % Find out the name of eNBs that are interfered by eNBname including
    % eNBname
    eNBInterfering = [eNBname;eNB(eNBix).Neighbor1stTier;];
    eNBInterfering(find(eNBInterfering==0)) = [];
    % Add the UE to the lists of eNBInterfering
    for i = 1:length(eNBInterfering)
        ix = find([eNB.NameTag]==eNBInterfering(i));
        eNB(ix).Interfering(find(eNB(ix).Interfering==0,1)) = ue;
    end
    
    % Do the same for Picos in the eNBs appeared in eNBInterfering
    if NumPico > 0
        for i = 1:length(eNBInterfering)
            ix = find(round(([Pico.NameTag]-floor([Pico.NameTag]/1000)*1000)*10)/10==eNBInterfering(i));
            for j = 1:length(ix)
                Pico(ix(j)).Interfering(find(Pico(ix(j)).Interfering==0,1)) = ue;
            end
        end
    end
end

% Record the CSB values of candidate stations for each UE
if NumPico > 0
    for i = 1:NumUE
        UE(i) = FetchCSB(eNB,UE(i),NumeNBPerCluster,NumPicoPerCell,Pico);
    end
else
    for i = 1:NumUE
        UE(i) = FetchCSB(eNB,UE(i),NumeNBPerCluster,NumPicoPerCell);
    end
end

% Combine eNB and Pico to get the structure Station
if NumPico > 0
    Station = [eNB;Pico];
else
    Station = eNB;
end
% Find out UE.InterfereIx
for i = 1:NumUE
    for j = 1:length(UE(i).Interfere)
        if UE(i).Interfere(j) ~= 0
            UE(i).InterfereIx(j) = find([Station.NameTag]==UE(i).Interfere(j));
        end
    end
end